library mattcorp_login;

import 'dart:async';
import 'package:flutter/material.dart';
import 'package:flutter_webview_plugin/flutter_webview_plugin.dart';

class MattCorpLogin {
  MattCorpLogin({this.client});
  final String client;

  final webview = new FlutterWebviewPlugin();

  Future<String> login(BuildContext context) async {
    // await _channel.invokeMethod('login');
    StreamController<String> sc = new StreamController();

    bool _closed = false;

    _close() {
      if (!_closed) {
        _closed = true;
        webview.close();
        Navigator.pop(context);
        sc.close();
        //print('Everything closed');
      }
    }

    webview.onUrlChanged.listen((String url) {
      print(url);
      if (url.startsWith('https://mobileauth.mattcorp')) {
        print('Found return URL');
        final Uri uri = Uri.parse(url);
        final String token = uri.queryParameters["token"];
        sc.add(token);
        //print('Test');
        webview.dispose();
        _close();
      }
    });

    webview.onStateChanged.listen((WebViewStateChanged state) {
      //print('State changed');
      print(state);
    });

    webview.onDestroy.listen((_) {
      //print('Destroyed');
      _close();
    });

    /*webview.launch(
      'https://id.mattcorp.com/jwt?embed=true&client=recipea&return=https%3A%2F%2Fmobileauth.mattcorp',
      //clearCache: true,
      clearCookies: true,
    );*/

    Navigator.push(context,
        new _NoAnimationsRoute(builder: (context) => new LoginScreen(sc, client)));

    //print('Should have opened');

    String code = await sc.stream
        .asBroadcastStream()
        .firstWhere((_) => true, orElse: () => null);
    //print('Code');
    //print(code);
    return code;
  }
}

class _NoAnimationsRoute<T> extends MaterialPageRoute<T> {
  _NoAnimationsRoute({WidgetBuilder builder}) : super(builder: builder);

  @override
  Widget buildTransitions(BuildContext context, Animation<double> animation,
          Animation<double> secondaryAnimation, Widget child) =>
      child;

  @override
  Duration get transitionDuration => const Duration(milliseconds: 1);
}

class LoginScreen extends StatefulWidget {
  const LoginScreen(this.sc, this.client, {Key key}) : super(key: key);
  final String client;
  final StreamController<String> sc;

  @override
  _LoginScreenState createState() => new _LoginScreenState(client);
}

class _LoginScreenState extends State<LoginScreen> {
  _LoginScreenState(this._client);
  final String _client;

  @override
  Widget build(BuildContext context) {
    return new WebviewScaffold(
      url:
          "https://id.mattcorp.com/jwt?embed=true&client=$_client&return=https%3A%2F%2Fmobileauth.mattcorp",
      appBar: new AppBar(
        automaticallyImplyLeading: false,
      ),
      clearCookies: true,
      clearCache: true,
    );
  }

  @override
  void dispose() {
    //print('State disposed');
    widget.sc.close();
    super.dispose();
  }
}
